@@ -173,7 +173,7 @@ func toIdentType(ctx *blockCtx, ident *ast.Ident) types.Type {
 	if t, ok := v.(*types.TypeName); ok {
 		return t.Type()
 	}
-	if v, _ := lookupPkgRef(ctx, nil, ident); v != nil {
+	if v, _ := lookupPkgRef(ctx, nil, ident, objPkgRef); v != nil {
 		if t, ok := v.(*types.TypeName); ok {
 			return t.Type()
 		}
@@ -209,15 +209,39 @@ func toStructType(ctx *blockCtx, v *ast.StructType) *types.Struct {
 	fieldList := v.Fields.List
 	fields := make([]*types.Var, 0, len(fieldList))
 	tags := make([]string, 0, len(fieldList))
+	names := make(map[string]token.Pos)
+	chkRedecl := func(name string, pos token.Pos) bool {
+		if name == "_" {
+			return false
+		}
+		if opos, ok := names[name]; ok {
+			npos := ctx.Position(pos)
+			ctx.handleCodeErrorf(&npos, "%v redeclared\n\t%v other declaration of %v",
+				name, ctx.Position(opos), name)
+			return true
+		}
+		names[name] = pos
+		return false
+	}
 	for _, field := range fieldList {
 		typ := toType(ctx, field.Type)
 		if field.Names == nil { // embedded
-			fld := types.NewField(token.NoPos, pkg, getTypeName(typ), typ, true)
+			name := getTypeName(typ)
+			if chkRedecl(name, field.Type.Pos()) {
+				continue
+			}
+			if t, ok := typ.(*types.Named); ok { // #1196: embedded type should ensure loaded
+				ctx.loadNamed(ctx.pkg, t)
+			}
+			fld := types.NewField(token.NoPos, pkg, name, typ, true)
 			fields = append(fields, fld)
 			tags = append(tags, toFieldTag(field.Tag))
 			continue
 		}
 		for _, name := range field.Names {
+			if chkRedecl(name.Name, name.NamePos) {
+				continue
+			}
 			fld := types.NewField(token.NoPos, pkg, name.Name, typ, false)
 			fields = append(fields, fld)
 			tags = append(tags, toFieldTag(field.Tag))
@@ -295,7 +319,11 @@ func toInterfaceType(ctx *blockCtx, v *ast.InterfaceType) types.Type {
 	var embeddeds []types.Type
 	for _, m := range methodsList {
 		if m.Names == nil { // embedded
-			embeddeds = append(embeddeds, toType(ctx, m.Type))
+			typ := toType(ctx, m.Type)
+			if t, ok := typ.(*types.Named); ok { // #1198: embedded type should ensure loaded
+				ctx.loadNamed(ctx.pkg, t)
+			}
+			embeddeds = append(embeddeds, typ)
 			continue
 		}
 		name := m.Names[0].Name
