@@ -19,17 +19,23 @@ package install
 
 import (
 	"fmt"
-	"github.com/qiniu/x/log"
+	"log"
 	"os"
+	"reflect"
+	"syscall"
 
+	"github.com/goplus/gop"
 	"github.com/goplus/gop/cl"
 	"github.com/goplus/gop/cmd/internal/base"
+	"github.com/goplus/gop/x/gocmd"
+	"github.com/goplus/gop/x/gopenv"
+	"github.com/goplus/gop/x/gopprojs"
 	"github.com/goplus/gox"
 )
 
-// Cmd - gop install
+// gop install
 var Cmd = &base.Command{
-	UsageLine: "gop install [-v] <GopPackages>",
+	UsageLine: "gop install [-v] [packages]",
 	Short:     "Build Go+ files and install target to GOBIN",
 }
 
@@ -43,20 +49,58 @@ func init() {
 }
 
 func runCmd(_ *base.Command, args []string) {
-	err := flag.Parse(base.SkipSwitches(args, flag))
+	err := flag.Parse(args)
 	if err != nil {
 		log.Fatalln("parse input arguments failed:", err)
 	}
-	ssargs := flag.Args()
-	dir, recursive := base.GetBuildDir(ssargs)
+
+	pattern := flag.Args()
+	if len(pattern) == 0 {
+		pattern = []string{"."}
+	}
+
+	projs, err := gopprojs.ParseAll(pattern...)
+	if err != nil {
+		log.Panicln("gopprojs.ParseAll:", err)
+	}
 
 	if *flagVerbose {
 		gox.SetDebug(gox.DbgFlagAll &^ gox.DbgFlagComments)
 		cl.SetDebug(cl.DbgFlagAll)
 		cl.SetDisableRecover(true)
 	}
-	base.GenGoForBuild(dir, recursive, func() { fmt.Fprintln(os.Stderr, "GenGo failed, stop installing") })
-	base.RunGoCmd(dir, "install", args...)
+
+	gopEnv := gopenv.Get()
+	conf := &gop.Config{Gop: gopEnv}
+	confCmd := &gocmd.Config{Gop: gopEnv}
+	for _, proj := range projs {
+		install(proj, conf, confCmd)
+	}
+}
+
+func install(proj gopprojs.Proj, conf *gop.Config, install *gocmd.InstallConfig) {
+	var obj string
+	var err error
+	switch v := proj.(type) {
+	case *gopprojs.DirProj:
+		obj = v.Dir
+		err = gop.InstallDir(obj, conf, install)
+	case *gopprojs.PkgPathProj:
+		obj = v.Path
+		err = gop.InstallPkgPath("", v.Path, conf, install)
+	case *gopprojs.FilesProj:
+		err = gop.InstallFiles(v.Files, conf, install)
+	default:
+		log.Panicln("`gop install` doesn't support", reflect.TypeOf(v))
+	}
+	if err == syscall.ENOENT {
+		fmt.Fprintf(os.Stderr, "gop install %v: not found\n", obj)
+	} else if err != nil {
+		fmt.Fprintln(os.Stderr, err)
+	} else {
+		return
+	}
+	os.Exit(1)
 }
 
 // -----------------------------------------------------------------------------
