@@ -1354,7 +1354,7 @@ func (p *parser) parseFuncTypeOrLit() ast.Expr {
 
 func (p *parser) isCommand(x ast.Expr) bool {
 	switch x.(type) {
-	case *ast.Ident, *ast.SelectorExpr:
+	case *ast.Ident, *ast.SelectorExpr, *ast.ErrWrapExpr:
 	default:
 		return false
 	}
@@ -1908,6 +1908,9 @@ L:
 			} else {
 				break L
 			}
+		case token.NOT, token.QUESTION:
+			x = &ast.ErrWrapExpr{X: x, Tok: p.tok, TokPos: p.pos}
+			p.next()
 		default:
 			if allowCmd && p.isCommand(x) {
 				if lhs {
@@ -1923,6 +1926,18 @@ L:
 	return x
 }
 
+// parseErrWrapExpr: expr! expr? expr?:defval
+func (p *parser) parseErrWrapExpr(lhs, allowTuple, allowCmd bool) ast.Expr {
+	x := p.parsePrimaryExpr(lhs, allowTuple, allowCmd)
+	if expr, ok := x.(*ast.ErrWrapExpr); ok {
+		if p.tok == token.COLON {
+			p.next()
+			expr.Default = p.parseUnaryExpr(false, true, false)
+		}
+	}
+	return x
+}
+
 // If lhs is set and the result is an identifier, it is not resolved.
 func (p *parser) parseUnaryExpr(lhs, allowTuple, allowCmd bool) ast.Expr {
 	if p.trace {
@@ -1993,27 +2008,6 @@ func (p *parser) parseUnaryExpr(lhs, allowTuple, allowCmd bool) ast.Expr {
 	return p.parseErrWrapExpr(lhs, allowTuple, allowCmd)
 }
 
-// parseErrWrapExpr: expr! expr? expr?:defval
-func (p *parser) parseErrWrapExpr(lhs, allowTuple, allowCmd bool) ast.Expr {
-	x := p.parsePrimaryExpr(lhs, allowTuple, allowCmd)
-	switch p.tok {
-	case token.NOT: // expr!
-		expr := &ast.ErrWrapExpr{X: x, Tok: token.NOT, TokPos: p.pos}
-		p.next()
-		return expr
-	case token.QUESTION: // expr? expr?:defval
-		expr := &ast.ErrWrapExpr{X: x, Tok: token.QUESTION, TokPos: p.pos}
-		p.next()
-		if p.tok == token.COLON {
-			p.next()
-			expr.Default = p.parseUnaryExpr(false, true, false)
-		}
-		return expr
-	default:
-		return x
-	}
-}
-
 func (p *parser) tokPrec() (token.Token, int) {
 	tok := p.tok
 	if p.inRHS && tok == token.ASSIGN {
@@ -2747,12 +2741,14 @@ func (p *parser) parseForPhraseStmtPart(lhs []ast.Expr) *ast.ForPhraseStmt {
 	tokPos := p.expect(token.ARROW) // <-
 	x := p.parseExpr(false, false, true)
 	var cond ast.Expr
+	var ifPos token.Pos
 	if p.tok == token.IF || p.tok == token.COMMA {
+		ifPos = p.pos
 		p.next()
 		cond = p.parseExpr(false, false, false)
 	}
 
-	stmt := &ast.ForPhraseStmt{ForPhrase: &ast.ForPhrase{TokPos: tokPos, X: x, Cond: cond}}
+	stmt := &ast.ForPhraseStmt{ForPhrase: &ast.ForPhrase{TokPos: tokPos, X: x, IfPos: ifPos, Cond: cond}}
 	switch len(lhs) {
 	case 1:
 		stmt.Value = p.toIdent(lhs[0])
@@ -2796,11 +2792,13 @@ func (p *parser) parseForPhrase() *ast.ForPhrase { // for k, v <- container if c
 	x := p.parseExpr(false, false, true)
 	var init ast.Stmt
 	var cond ast.Expr
+	var ifPos token.Pos
 	if p.tok == token.IF || p.tok == token.COMMA { // `condition` or `init; condition`
+		ifPos = p.pos
 		p.next()
 		init, cond = p.parseForPhraseCond()
 	}
-	return &ast.ForPhrase{For: pos, Key: k, Value: v, TokPos: tokPos, X: x, Init: init, Cond: cond}
+	return &ast.ForPhrase{For: pos, Key: k, Value: v, TokPos: tokPos, X: x, IfPos: ifPos, Init: init, Cond: cond}
 }
 
 func (p *parser) parseForStmt() ast.Stmt {
